/* ------------------------------------------------------------------------
   file:   linked_list.h - a self verifying list ADT - public header
   ------------------------------------------------------------------------ */
#ifndef _C_LINKED_LIST_H_
#define _C_LINKED_LIST_H_

#ifndef TRUE
#  define TRUE 1
#endif 

#ifndef FALSE
#  define FALSE 0
#endif

#ifdef __cplusplus
extern "C" {
#endif
/***********************************************************************
 * Forward Singly Linked List libary                                   *
 * ==========================                                          *
 * Using the list:                                                     *
 * 1.) call list_create() to construct a list                          *
 * 2.) use the provided functionality for list                         *
 * 3.) call list_destroy() when finished                               *
 *                                                                     *
 * This list may not contain null elements as it will break the        *
 * iteration functions.                                                *
 ***********************************************************************/


/***********************************************************************
 * internally used structures                                          *
 ***********************************************************************/
typedef struct list_node_tag {
  int magic;
  void *data;                  /* pointer to user data */
  struct list_node_tag *next;  /* pointer to next node */
} list_node;

typedef struct linked_list_tag {
  list_node root;              /* root node, never holds data */
  list_node *tail;             /* tail node, for faster tail additions */
  list_node *current;          /* current pointer in list for iteration */
  unsigned int count;          /* total number of user data elements */
  const char *lastfunc;

} _llist, *linked_list;

/***********************************************************************
 * library functions                                                   *
 ***********************************************************************/
/** construct a list */
linked_list list_create(void);
/** destroy a list */
void list_destroy(linked_list list);
/** add a data element to the end of the list*/
void list_add_tail(linked_list list, void *data);
/** add a data element to the front of the list */
void list_add_head(linked_list list, void *data);
/** returns TRUE if the element exists in the list */
int list_contains(linked_list list, void* data);
/** removes an element from the list, returns TRUE
   if the element existed in the list*/
int list_remove(linked_list list, void* data);
/** remove the head of the list and returns it
 * if there is no more elements, returns NULL
 */
void* list_remove_head(linked_list list);
/** remove the tail of the list and returns it
 * if there is no more elements, returns NULL
 */
void* list_remove_tail(linked_list list);
/** iterate through the list by fcn callback.
   the callback function should return TRUE to continue
   the iteration. or FALSE to end the iteration
   userdata is a user defined convenience value
*/
typedef void listcallback_data(void* data, void *userdata);
void list_enumerate_data(linked_list list, listcallback_data callback, void *userdata);
/** iterate through the list by fcn callback.
   the callback function should return TRUE to continue
   the iteration. or FALSE to end the iteration */
typedef void listcallback(void* data);
void list_enumerate(linked_list list, listcallback callback);
/** gets the next item in the list. behaves as if the list is circular. returns NULL when there are no more items in the list
*/
void* list_next(linked_list list);
/** returns the number of elements in the list */
unsigned int list_count(linked_list list);
/** clears the list */
void list_clear(linked_list list);

/***********************************************************************
 * for internal use only                                               *
 ***********************************************************************/
/** creates a node */
list_node* list_create_node(void *data);

#ifdef __cplusplus
}
#endif

#endif /* _LIST_H_ */
